
'''various contants'''

### P2TH tags for Peercoin mainnet/testnet ### ### ### 

## mainnet
mainnet_PAPROD = "U624wXL6iT7XZ9qeHsrtPGEiU78V1YxDfwq75Mymd61Ch56w47KE"
mainnet_PAPROD_addr = "PAprodbYvZqf4vjhef49aThB9rSZRxXsM6"
mainnet_PATEST = "UAbxMGQQKmfZCwKXAhUQg3MZNXcnSqG5Z6wAJMLNVUAcyJ5yYxLP"
mainnet_PATEST_addr = "PAtesth4QreCwMzXJjYHBcCVKbC4wjbYKP"

## testnet
testnet_PAPROD = "cTJVuFKuupqVjaQCFLtsJfG8NyEyHZ3vjCdistzitsD2ZapvwYZH"
testnet_PAPROD_addr = "miHhMLaMWubq4Wx6SdTEqZcUHEGp8RKMZt"
testnet_PATEST = "cQToBYwzrB3yHr8h7PchBobM3zbrdGKj2LtXqg7NQLuxsHeKJtRL"
testnet_PATEST_addr = "mvfR2sSxAfmDaGgPcmdsTwPqzS6R9nM5Bo"

### ### ### ### ### ### ### ### ### ### ### ### ### ###

### P2TH fee

mainnet_p2th_fee = float(0.01)
testnet_p2th_fee = float(0.01)

